#!/usr/bin/python3
# encoding: utf-8

'''
This script will pull the solar production data from Domoticz and send it to pvoutput.org
You can specify up to 3 devices which will be added up.

Add it to crontab like this example, to run every 5 minutes (can be 1 minute in donation mode):
*/5 * * * * python3 /home/pi/domoticz/scripts/pvoutput_aggregate_perday.py -d1 24 -d2 65 -d3 109

- The Domotics variables are parsed via arguments into the script: -a () -d1 and optional d2, d3).

The d1, d2 and d3 values are the devices Idx, which you find in Domoticz under Setup -> Devices. The options d2 and d3 are optional.

- The PVoutput variables need to be edited in the script below at PV_OUTPUT_SYSTEMID and PV_OUTPUT_APIKEY 

- Your location, the longitude and latitude variables need to be entered in the script. This is needed for calculating your sunrise/sunset times.


This version v3 uses daily production counters (previous versions used cumulative counters).
Changes:
- ['result'][0]['Data'] became ['result'][0]['CounterToday']
- c1': "1" removed
- added a check to not run during the night, between 22:30 and 5:00 local time (could be removed due to the next check)
- added a check to only run after sunrise (minus 10 minutes) and before sunset (plus 30 minutes).
- added reading and sending the power value as well.

Domoticz values van be tested via browser like this example:
http://192.168.2.246:8080/json.htm?type=devices&rid=109

Version v3

'''

import sys, json
from urllib.request import urlopen
import requests
from datetime import datetime, date, timezone, timedelta
from argparse import ArgumentParser
from suntime import Sun, SunTimeException


# ENTER YOUR PVOUTPUT.ORG VALUES HERE:

PV_OUTPUT_SYSTEMID = 'xxxxx'
PV_OUTPUT_APIKEY = 'xxxxxxxxxxxxxxxxx'
PVOUTPUT_URL = 'http://pvoutput.org/service/r2/addstatus.jsp'

# needed for calculating sunrise/sunset times
latitude = 52.05
longitude = 5.23




def getDateStringOfToday():
    return date.today().strftime("%Y%m%d");


def getTimeString():
    return datetime.now().strftime("%H:%M")


def sendUpdateToPVOutput(energystring, powerstring):
    pvoutputdata = {
      'd': getDateStringOfToday(),
      't': getTimeString(),
      'v1': energystring,
      'v2': powerstring
    }

    headerspv = {
        'X-Pvoutput-SystemId': PV_OUTPUT_SYSTEMID,
        'X-Pvoutput-Apikey': PV_OUTPUT_APIKEY
    }

    responsepv = requests.post(PVOUTPUT_URL, headers=headerspv, data=pvoutputdata, timeout=10)

    #print ("Response: " + responsepv.text + "Energystring: " + energystring)

    return responsepv


def checkIfSunIsUp():
    sun = Sun(latitude, longitude)

    # Get today's sunrise and sunset in UTC
    today_sr = sun.get_sunrise_time()
    today_ss = sun.get_sunset_time()

    # 30 minutes before/after sunrise/sunset
    today_sr_threshold = today_sr - timedelta(minutes=10)
    today_ss_threshold = today_ss + timedelta(minutes=30)

    # make string
    starttime_utc = int(today_sr_threshold.strftime('%H%M'))
    stoptime_utc = int(today_ss_threshold.strftime('%H%M'))

    # get current time in utc
    now_utc = datetime.now(timezone.utc)
    nowtime_utc = int(now_utc.strftime('%H%M'))

    #debug:
    # print ("nowtime_utc is " + str(nowtime_utc))
    # print ("starttime_utc is " + str(starttime_utc))
    # print ("stoptime_utc is " + str(stoptime_utc))

    # make decision if sun is up
    if nowtime_utc > starttime_utc and nowtime_utc < stoptime_utc:
        #print ("Sun is up")
        return "yes"
    else:
        #print ("Sun is not up")
        return "no"




def main(argv=None):

    parser = ArgumentParser(description='Upload solar data from Domoticz to pvoutput.org for up to three devices added up')
    parser.add_argument("-u", "--url", dest="url", help="URL to Domoticz, eg: http://localhost:8080", default='http://localhost:8080')
    parser.add_argument("-d1", "--device-id1", dest="device1", help="Device id voor de P1 Gas Smart Meter", type=int, required=True)
    parser.add_argument("-d2", "--device-id2", dest="device2", help="Device id voor de P1 Gas Smart Meter", type=int)
    parser.add_argument("-d3", "--device-id3", dest="device3", help="Device id voor de P1 Gas Smart Meter", type=int)

    args = parser.parse_args()


    # Do not run at night
    time = datetime.now().strftime("%H%M")
    time = int(time)
    if time > 2230 or time < 500:
        sys.exit()

    # Do not run before sunrise and after sunset, with 30 mins extra after sunset, to be sure not to miss the final data end of day

    sunUpCheck = checkIfSunIsUp()

    if sunUpCheck == "no":
        #print ("sun not up, quit")
        sys.exit()



    # collect data from Domoticz

    device_data = json.load(urlopen("%s/json.htm?type=devices&rid=%s" % (args.url, args.device1), timeout=5))

    #Counter
    count_agg = device_data['result'][0]['CounterToday']
    count_agg = count_agg[:-4]
    count_agg = float(count_agg)

    #Power
    power_agg = device_data['result'][0]['Usage']
    power_agg = power_agg[:-5]
    power_agg = float(power_agg)


    if args.device2 is not None:
      device_data = json.load(urlopen("%s/json.htm?type=devices&rid=%s" % (args.url, args.device2), timeout=5))
      count2 = device_data['result'][0]['CounterToday']
      count2 = count2[:-4]
      count_agg = float(count2) + count_agg

      power2 = device_data['result'][0]['Usage']
      power2 = power2[:-5]
      power_agg = float(power2) + power_agg


    if args.device3 is not None:
      device_data = json.load(urlopen("%s/json.htm?type=devices&rid=%s" % (args.url, args.device3), timeout=5))
      count3 = device_data['result'][0]['CounterToday']
      count3 = count3[:-4]
      count_agg = float(count3) + count_agg

      power3 = device_data['result'][0]['Usage']
      power3 = power3[:-5]
      power_agg = float(power3) + power_agg
    

    if count_agg >= 0:
      count_agg = int(count_agg * 1000)
      count_agg = str(count_agg)

      power_agg = int(power_agg)
      power_agg = str(power_agg)


      #print(count_agg)
      #print(power_agg)

      ### uncomment to send the data to pvoutput.org ###
      response = sendUpdateToPVOutput(count_agg, power_agg)





if __name__ == "__main__":
    sys.exit(main())
